MODULE mod_phys_lmdz_ensemble
  
  LOGICAL,SAVE :: ensemble_management         ! true if ensemble is managed
                                              ! that's mean one additionnal dimension is added to output
  INTEGER,SAVE :: COMM_LMDZ_PHY_ENSEMBLE      ! MPI communicator for the ensemble pool
  INTEGER,SAVE :: whole_ensemble_size         ! number of members in whole ensemble
  INTEGER,SAVE :: whole_ensemble_rank         ! local member number in whole ensemble
  
  
CONTAINS

  SUBROUTINE init_phys_lmdz_ensemble(ensemble_communicator, member_communicator)
  USE ioipsl_getin_p_mod
  IMPLICIT NONE
    INTEGER, INTENT(IN)  :: ensemble_communicator          ! communicator of pool ensemble
    INTEGER, INTENT(OUT) :: member_communicator            ! returned communicator for local member
    INTEGER              :: ensemble_comm_size
    INTEGER              :: ensemble_comm_rank
    INTEGER              :: ensemble_pool_rank
    INTEGER              :: ensemble_pool_size
    INTEGER              :: member_size
    INTEGER              :: ensemble_rank
    INTEGER              :: ierr
 
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  
  !! SPLIT COMMUNICATOR FOR ENSEMBLE         !!!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     COMM_LMDZ_PHY_ENSEMBLE = ensemble_communicator
     ensemble_pool_size = 1
     CALL getin("ensemble_pool_size", ensemble_pool_size)
     whole_ensemble_size=ensemble_pool_size
     CALL getin("whole_ensemble_size", whole_ensemble_size)
     ensemble_management=.FALSE.
     IF (whole_ensemble_size>1) ensemble_management=.TRUE.  
     ensemble_pool_rank=0
     CALL getin("ensemble_pool_rank", ensemble_pool_rank)
     
     CALL MPI_COMM_SIZE(COMM_LMDZ_PHY_ENSEMBLE,ensemble_comm_size,ierr)
     CALL MPI_COMM_RANK(COMM_LMDZ_PHY_ENSEMBLE,ensemble_comm_rank,ierr)
     PRINT*,ensemble_comm_size,ensemble_comm_rank
     IF (MOD(ensemble_comm_size,ensemble_pool_size)/=0) THEN
      CALL abort_physic('Init_mpi','Nb of MPI processes in whole ensemble is not divisible by the size of ensemble',1)
     ENDIF  
     member_size = ensemble_comm_size/ensemble_pool_size
     ensemble_rank = ensemble_comm_rank/member_size
     CALL MPI_COMM_SPLIT(COMM_LMDZ_PHY_ENSEMBLE, ensemble_rank, ensemble_comm_rank, member_communicator,ierr)
     whole_ensemble_rank = ensemble_pool_rank*ensemble_pool_size + ensemble_rank       
   
   END SUBROUTINE  init_phys_lmdz_ensemble


END MODULE mod_phys_lmdz_ensemble
